@@ -1,22 +1,24 @@
-from functools import partial
-from logging import warning
-from re import compile as rc, findall
-from datetime import datetime
-from json import load
-from io import BytesIO, StringIO
-from typing import Union
 from ast import literal_eval
+from datetime import datetime
+from functools import partial
+from io import BytesIO, StringIO
+from json import load
+from logging import warning
+from os.path import abspath
+from re import compile as rc, findall
+from typing import Union, TypedDict
 
 
-from requests import Session
 from jdatetime import datetime as jdatetime
 from pandas import read_csv, to_numeric, DataFrame, read_html, to_datetime
+from requests import Session
 
 
 strptime = datetime.strptime
 jstrptime = jdatetime.strptime
+j_ymd_parse = partial(jstrptime, format='%Y/%m/%d')
 GET = Session().get
-
+DB_PATH = abspath(f'{__file__}/../database/ids.json')
 
 FARSI_NORM = ''.maketrans('يك', 'یک')
 F = r'(-?\d+(?:\.\d+)?)'  # float pattern
@@ -65,9 +67,9 @@ STR_TO_NUM = partial(rc(rf"'{F}'").sub, r'\1')
 INDEX_CHANGE_MATCH = rc(rf"<div[^>]*>(\()?{F}\)?</div>(?: {F}%)?").match
 INDEX_TIMESTAMP_MATCH = rc(r'(\d\d)/(\d+)/(\d+) (\d\d):(\d\d):(\d\d)').match
 
-with open(f'{__file__}/../ids.json', encoding='utf8') as f:
-    KNOWN_IDS: dict[str, str] = load(f)
-KNOWN_IDS |= {v[0]: v for v in KNOWN_IDS.values()}
+with open(DB_PATH, encoding='utf8') as f:
+    L18S: dict[str, str] = load(f)
+INSCODES = {v[0]: v for v in L18S.values()}
 
 
 def get_content(url) -> bytes:
@@ -79,6 +81,35 @@ def fa_norm_text(url) -> str:
     return get_content(url).decode().translate(FARSI_NORM)
 
 
+class MarketWatchInitDict(TypedDict, total=False):
+    dataframe: DataFrame
+    market_last_transaction: jdatetime
+    tset_status: str
+    tset_index: float
+    tset_index_change: float
+    tse_tvol: float
+    tse_tval: float
+    tse_tno: float
+    otc_status: str
+    otc_tvol: float
+    otc_tval: float
+    otc_tno: int
+    derivatives_status: int
+
+
+class IntraDayDict(TypedDict, total=False):
+    general: dict
+    thresholds: DataFrame
+    closings: DataFrame
+    candles: DataFrame
+    states: DataFrame
+    trades: DataFrame
+    holders: DataFrame
+    yesterday_holders: DataFrame
+    client_types: dict[str, int]
+    best_limits: DataFrame
+
+
 class Instrument:
 
     __slots__ = 'ins_code', 'l18', 'l30'
@@ -100,9 +131,9 @@ class Instrument:
         return self.ins_code
 
     @staticmethod
-    def from_l18(l18: str, /):
+    def from_l18(l18: str, /) -> 'Instrument':
         try:
-            ins_code, _, l30 = KNOWN_IDS[l18]
+            ins_code, _, l30 = L18S[l18]
         except KeyError:
             raise KeyError(
                 'l18 not found in KNOWN_IDS, try Instrument.from_search')
@@ -238,7 +269,12 @@ class Instrument:
             , index_col='date', parse_dates=True , dtype='int64', low_memory=False)
 
     def get_identification(self) -> DataFrame:
-        """Return the information available in the identification (شناسه) tab."""
+        """Return the information available in the identification (شناسه) tab.
+
+        Related API descriptions:
+            https://cdn.tsetmc.com/Site.aspx?ParTree=1114111118&LnkIdn=83
+            http://en.tsetmc.com/Site.aspx?ParTree=111411111Z
+        """
         text = fa_norm_text(f'http://www.tsetmc.com/Loader.aspx?Partree=15131M&i={self.ins_code}')
         return read_html(text, index_col=0)[0]
 
@@ -312,8 +348,12 @@ class Instrument:
         yesterday_holders=False,
         client_types=True,
         best_limits=True,
-    ):
-        """Get intraday info for the given date in YYYYMMDD format."""
+    ) -> IntraDayDict:
+        """Get intraday info for the given date in YYYYMMDD format.
+
+        For the meaning of instrument state codes refer to
+            http://en.tsetmc.com/Site.aspx?ParTree=111411111Y
+        """
         text = fa_norm_text(f'http://www.tsetmc.com/Loader.aspx?ParTree=15131P&i={self.ins_code}&d={date}')
         find = text.find
         find_start = 0
@@ -411,11 +451,12 @@ class Instrument:
     def get_adjustments(self) -> DataFrame:
         text = fa_norm_text(f'http://www.tsetmc.com/Loader.aspx?Partree=15131G&i={self.ins_code}')
         df = read_html(text)[0]
-        df['تاریخ'] = df['تاریخ'].apply(partial(jstrptime, format='%Y/%m/%d'))
+        df.columns = ('date', 'adj_pc', 'pc')
+        df['date'] = df['date'].apply(j_ymd_parse)
         return df
 
 
-def get_market_watch_init(index=False) -> dict:
+def get_market_watch_init(index=False) -> MarketWatchInitDict:
     """Return the market status which are the info used in creating filters.
 
     For more information about filters see:
@@ -558,3 +599,16 @@ def _parse_index(s: str) -> dict:
     if (m3 := index_change_match[3]) is not None:
         result['tse_index_change_percent'] = float(m3)
     return result
+
+
+def get_price_adjustments(flow: int) -> DataFrame:
+    """Get price adjustments for a particular flow.
+
+    Related APIs:
+        http://cdn.tsetmc.com/Site.aspx?ParTree=1114111124&LnkIdn=843
+    """
+    text = fa_norm_text(f'http://tsetmc.com/Loader.aspx?Partree=151319&Flow={flow}')
+    df = read_html(text)[0]
+    df.columns = ('l18', 'l30', 'date', 'adj_pc', 'pc')
+    df['date'] = df['date'].apply(j_ymd_parse)
+    return df
